/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#include "casp_udio_comm.h"

#define CASP_START_BYTE             0x74
#define CASP_HEADER_SIZE            4
#define CASP_CHECKSUM_IDX           3

#define CASP_DATA_RECV_TIMEOUT      5000

CaspUdioComm::CaspUdioComm()
{
    m_send_buffer = m_recv_buffer = 0;
    m_send_buffer_size = m_recv_buffer_size = 0;
    m_recvd_size_fp = 0;
    m_recv_data_fp = 0;
    m_send_data_fp = 0;
    m_data_received = 0;
    m_prev_available_bytes = 0;
    m_prev_sys_clk = 0;
}

CaspUdioComm::~CaspUdioComm()
{
}

bool CaspUdioComm::Initialize(int send_buffer_size, int recv_buffer_size, bool send_recv_buffers_common)
{
    if(send_buffer_size < 1 && recv_buffer_size < 1)
        return false;
    //reset previous
    if(m_send_buffer) free(m_send_buffer);
    if(m_recv_buffer && (m_recv_buffer != m_send_buffer)) free(m_recv_buffer);
    //
    m_send_buffer_size = send_buffer_size+CASP_HEADER_SIZE;
    m_recv_buffer_size = recv_buffer_size+CASP_HEADER_SIZE;
    if(send_buffer_size != recv_buffer_size)
        send_recv_buffers_common = false;
    if(send_recv_buffers_common)
    {
        m_send_buffer = (unsigned char*)malloc(m_send_buffer_size);
        m_recv_buffer = m_send_buffer;
    }
    else
    {
        m_send_buffer = (unsigned char*)malloc(m_send_buffer_size);
        m_recv_buffer = (unsigned char*)malloc(m_recv_buffer_size);
    }
    return true;
}

unsigned char* CaspUdioComm::GetSendBuffer()
{
    return m_send_buffer+CASP_HEADER_SIZE;
}

unsigned char* CaspUdioComm::GetRecvBuffer()
{
    return m_recv_buffer+CASP_HEADER_SIZE;
}

void CaspUdioComm::SetDataRecvdSizeCallback(casp_udio_comm_recvd_size fp)
{
    m_recvd_size_fp = fp;
}

void CaspUdioComm::SetRecvDataCallback(casp_udio_comm_recv_data fp)
{
    m_recv_data_fp = fp;
}

void CaspUdioComm::SetSendDataCallback(casp_udio_comm_send_data fp)
{
    m_send_data_fp = fp;
}

void CaspUdioComm::SendData()
{
    m_send_buffer[0] = CASP_START_BYTE;
    m_send_buffer[1] = (m_send_buffer_size-CASP_HEADER_SIZE) & 0xff;//lsb of size
    m_send_buffer[2] = ((m_send_buffer_size-CASP_HEADER_SIZE)>>8) & 0xff;
    unsigned char cal_checksum = 0;
    for(unsigned int i=CASP_HEADER_SIZE; i<m_send_buffer_size; ++i)
        cal_checksum += m_send_buffer[i];
    m_send_buffer[CASP_CHECKSUM_IDX] = cal_checksum;
    m_send_data_fp(m_send_buffer, m_send_buffer_size);
}

void CaspUdioComm::RecvData(long long current_system_clock)
{
    if(m_recvd_size_fp == 0 || m_recv_data_fp == 0)
        return;
    if(m_data_received == true)//return till user resets this variable
        return;
    int available_bytes = m_recvd_size_fp();
    //receive data
    if(available_bytes == (int)m_recv_buffer_size)
    {
        m_recv_data_fp(m_recv_buffer, available_bytes);
        //validate received data with checksum
        if(m_recv_buffer[0] == CASP_START_BYTE)
        {
            unsigned char cal_checksum = 0;
            for(unsigned int i=CASP_HEADER_SIZE; i<m_recv_buffer_size; ++i)
                cal_checksum += m_recv_buffer[i];
            if(cal_checksum == m_recv_buffer[CASP_CHECKSUM_IDX])
                m_data_received = true;
            m_prev_available_bytes = 0;
            m_prev_sys_clk = current_system_clock;
        }
        else
        {
            m_prev_available_bytes = 0;
            m_prev_sys_clk = current_system_clock;
        }
    }
    else if(available_bytes > 0)
    {
        if((int)m_prev_available_bytes == available_bytes)
        {
            if((current_system_clock-m_prev_sys_clk) > CASP_DATA_RECV_TIMEOUT)
            {
                m_recv_data_fp(m_recv_buffer, available_bytes);
                m_prev_sys_clk = current_system_clock;
                m_prev_available_bytes = 0;
            }
        }
        else
            m_prev_sys_clk = current_system_clock;
        m_prev_available_bytes = available_bytes;
    }
}
