/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
/* This class can be used to transfer data to and from CASP native application running on PC.
 * It uses CASP UDIO data format that consists of 4 byte header followed by user data. User data shall be in multiples of 4 bytes.
 * 4byte data can be 32bit integer or float. Two 4byte data can be combined to form 64bit integer or double data types.
 * Header Format:
 * byte 0: Start byte with value 0x74
 * byte 1: user data size lower byte
 * byte 2: user data size higher byte
 * byte 3: user data checksum
 * byte 4 to end: user data
*/
//no dedicated buffer is used to save temporary send and receive data
//relies on internal buffers of the communication interface

#ifndef CASP_UDIO_COMM_H
#define CASP_UDIO_COMM_H

#include "stdlib.h"

typedef int (*casp_udio_comm_recvd_size)();
typedef int (*casp_udio_comm_recv_data)(unsigned char *rbuffer, int size_to_recv);
typedef int (*casp_udio_comm_send_data)(unsigned char *data, int data_size);

class CaspUdioComm
{
public:
    CaspUdioComm();
    ~CaspUdioComm();
    //call below methods during initialization (setup())
    bool Initialize(int send_buffer_size, int recv_buffer_size, bool send_recv_buffers_common = true);
    unsigned char *GetSendBuffer();
    unsigned char *GetRecvBuffer();
    void SetDataRecvdSizeCallback(casp_udio_comm_recvd_size fp);
    void SetRecvDataCallback(casp_udio_comm_recv_data fp);
    void SetSendDataCallback(casp_udio_comm_send_data fp);
    //call below methods during run time (loop())
    void RecvData(long long current_system_clock);
    bool IsDataRecvd(){return m_data_received;}
    void ResetDataRecvd(){m_data_received = 0; m_prev_available_bytes = 0; m_prev_sys_clk = 0;}
    void SendData();
private:
    unsigned char *m_send_buffer;
    unsigned char *m_recv_buffer;
    unsigned int m_send_buffer_size, m_recv_buffer_size, m_prev_available_bytes;
    long long m_prev_sys_clk;
    bool m_data_received;
    //
    casp_udio_comm_recvd_size m_recvd_size_fp;
    casp_udio_comm_recv_data m_recv_data_fp;
    casp_udio_comm_send_data m_send_data_fp;
};

#endif//CASP_UDIO_COMM_H
